# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'submit_bug.ui'
#
# Created: Mon Feb 20 23:09:17 2017
#      by: pyside-uic 0.2.15 running on PySide 1.2.4
#
# WARNING! All changes made in this file will be lost!

from PySide import QtCore, QtGui

class Ui_SubmitBug(object):
    def setupUi(self, SubmitBug):
        SubmitBug.setObjectName("SubmitBug")
        SubmitBug.resize(652, 594)
        self.gridLayout = QtGui.QGridLayout(SubmitBug)
        self.gridLayout.setObjectName("gridLayout")
        self.label = QtGui.QLabel(SubmitBug)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.horizontalLayout_2 = QtGui.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_4 = QtGui.QLabel(SubmitBug)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_2.addWidget(self.label_4)
        self.lineEdit_username = QtGui.QLineEdit(SubmitBug)
        self.lineEdit_username.setObjectName("lineEdit_username")
        self.horizontalLayout_2.addWidget(self.lineEdit_username)
        self.label_5 = QtGui.QLabel(SubmitBug)
        self.label_5.setObjectName("label_5")
        self.horizontalLayout_2.addWidget(self.label_5)
        self.lineEdit_password = QtGui.QLineEdit(SubmitBug)
        self.lineEdit_password.setEchoMode(QtGui.QLineEdit.Password)
        self.lineEdit_password.setObjectName("lineEdit_password")
        self.horizontalLayout_2.addWidget(self.lineEdit_password)
        self.gridLayout.addLayout(self.horizontalLayout_2, 2, 0, 1, 1)
        self.verticalLayout_3 = QtGui.QVBoxLayout()
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.label_6 = QtGui.QLabel(SubmitBug)
        self.label_6.setObjectName("label_6")
        self.verticalLayout_3.addWidget(self.label_6)
        self.comboBox_repo = QtGui.QComboBox(SubmitBug)
        self.comboBox_repo.setObjectName("comboBox_repo")
        self.comboBox_repo.addItem("")
        self.comboBox_repo.addItem("")
        self.comboBox_repo.addItem("")
        self.comboBox_repo.addItem("")
        self.verticalLayout_3.addWidget(self.comboBox_repo)
        self.gridLayout.addLayout(self.verticalLayout_3, 4, 0, 1, 1)
        self.verticalLayout = QtGui.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.label_2 = QtGui.QLabel(SubmitBug)
        self.label_2.setObjectName("label_2")
        self.verticalLayout.addWidget(self.label_2)
        self.lineEdit_summary = QtGui.QLineEdit(SubmitBug)
        self.lineEdit_summary.setObjectName("lineEdit_summary")
        self.verticalLayout.addWidget(self.lineEdit_summary)
        self.gridLayout.addLayout(self.verticalLayout, 5, 0, 1, 1)
        self.verticalLayout_2 = QtGui.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.label_3 = QtGui.QLabel(SubmitBug)
        self.label_3.setObjectName("label_3")
        self.verticalLayout_2.addWidget(self.label_3)
        self.plainTextEdit_details = QtGui.QPlainTextEdit(SubmitBug)
        self.plainTextEdit_details.setObjectName("plainTextEdit_details")
        self.verticalLayout_2.addWidget(self.plainTextEdit_details)
        self.gridLayout.addLayout(self.verticalLayout_2, 6, 0, 1, 1)
        self.checkBox_show_this_dialog = QtGui.QCheckBox(SubmitBug)
        self.checkBox_show_this_dialog.setChecked(True)
        self.checkBox_show_this_dialog.setObjectName("checkBox_show_this_dialog")
        self.gridLayout.addWidget(self.checkBox_show_this_dialog, 7, 0, 1, 1)
        self.horizontalLayout = QtGui.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.pushButton_cancel = QtGui.QPushButton(SubmitBug)
        self.pushButton_cancel.setObjectName("pushButton_cancel")
        self.horizontalLayout.addWidget(self.pushButton_cancel)
        self.pushButton_submit = QtGui.QPushButton(SubmitBug)
        self.pushButton_submit.setDefault(True)
        self.pushButton_submit.setFlat(False)
        self.pushButton_submit.setObjectName("pushButton_submit")
        self.horizontalLayout.addWidget(self.pushButton_submit)
        self.gridLayout.addLayout(self.horizontalLayout, 8, 0, 1, 1)
        spacerItem1 = QtGui.QSpacerItem(20, 20, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Fixed)
        self.gridLayout.addItem(spacerItem1, 3, 0, 1, 1)
        spacerItem2 = QtGui.QSpacerItem(20, 20, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Fixed)
        self.gridLayout.addItem(spacerItem2, 1, 0, 1, 1)

        self.retranslateUi(SubmitBug)
        QtCore.QMetaObject.connectSlotsByName(SubmitBug)

    def retranslateUi(self, SubmitBug):
        SubmitBug.setWindowTitle(QtGui.QApplication.translate("SubmitBug", "Submit Bug", None, QtGui.QApplication.UnicodeUTF8))
        self.label.setText(QtGui.QApplication.translate("SubmitBug", "<html><head/><body>\n"
"<p>Please provide as much information as possible about wath caused the bug that you are reporting.</p>\n"
"<p>This feature could recollect extra information about system, like OS, architecture, PYTHONPATH,<br>username, etc.</p></body></html>", None, QtGui.QApplication.UnicodeUTF8))
        self.label_4.setText(QtGui.QApplication.translate("SubmitBug", "GitHub username:", None, QtGui.QApplication.UnicodeUTF8))
        self.label_5.setText(QtGui.QApplication.translate("SubmitBug", "GitHub password", None, QtGui.QApplication.UnicodeUTF8))
        self.label_6.setText(QtGui.QApplication.translate("SubmitBug", "Repository:", None, QtGui.QApplication.UnicodeUTF8))
        self.comboBox_repo.setItemText(0, QtGui.QApplication.translate("SubmitBug", "pinguino-ide", None, QtGui.QApplication.UnicodeUTF8))
        self.comboBox_repo.setItemText(1, QtGui.QApplication.translate("SubmitBug", "pinguino-bootloaders", None, QtGui.QApplication.UnicodeUTF8))
        self.comboBox_repo.setItemText(2, QtGui.QApplication.translate("SubmitBug", "pinguino-libraries", None, QtGui.QApplication.UnicodeUTF8))
        self.comboBox_repo.setItemText(3, QtGui.QApplication.translate("SubmitBug", "pinguino-compilers", None, QtGui.QApplication.UnicodeUTF8))
        self.label_2.setText(QtGui.QApplication.translate("SubmitBug", "Summary:", None, QtGui.QApplication.UnicodeUTF8))
        self.label_3.setText(QtGui.QApplication.translate("SubmitBug", "Details:", None, QtGui.QApplication.UnicodeUTF8))
        self.checkBox_show_this_dialog.setText(QtGui.QApplication.translate("SubmitBug", "Show this dialog on unexpected errors.", None, QtGui.QApplication.UnicodeUTF8))
        self.pushButton_cancel.setText(QtGui.QApplication.translate("SubmitBug", "Cancel", None, QtGui.QApplication.UnicodeUTF8))
        self.pushButton_submit.setText(QtGui.QApplication.translate("SubmitBug", "Submit Now", None, QtGui.QApplication.UnicodeUTF8))

